==> rewrite-kubernetes-1.6.0-sources.jar: org/openrewrite/kubernetes/resource/FindExceedsResourceValue.java <==
/*
 * Copyright 2021 the original author or authors.
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * <p>
 * https://www.apache.org/licenses/LICENSE-2.0
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.openrewrite.kubernetes.resource;

import lombok.EqualsAndHashCode;
import lombok.Value;
import org.openrewrite.*;
import org.openrewrite.internal.lang.Nullable;
import org.openrewrite.yaml.YamlIsoVisitor;
import org.openrewrite.yaml.search.YamlSearchResult;
import org.openrewrite.yaml.tree.Yaml;

import static org.openrewrite.Tree.randomId;
import static org.openrewrite.kubernetes.tree.K8S.ResourceLimits.inLimits;
import static org.openrewrite.kubernetes.tree.K8S.ResourceLimits.inRequests;
import static org.openrewrite.kubernetes.tree.K8S.asResourceLimits;

@Value
@EqualsAndHashCode(callSuper = true)
public class FindExceedsResourceValue extends Recipe {

    @Option(displayName = "Resource value type",
            description = "The type of resource value to search for.",
            example = "limits",
            valid = {"limits", "requests"})
    String resourceValueType;

    @Option(displayName = "Resource limit type",
            description = "The type of resource limit to search for.",
            example = "memory",
            valid = {"cpu", "memory"})
    String resourceType;

    @Option(displayName = "Resource limit",
            description = "The resource limit maximum to search for to find resources that request more than the maximum.",
            example = "2Gi")
    String resourceLimit;

    @Option(displayName = "Optional file matcher",
            description = "Matching files will be modified. This is a glob expression.",
            required = false,
            example = "**/pod-*.yml")
    @Nullable
    String fileMatcher;

    @Override
    public String getDisplayName() {
        return "Find exceeds resource limit";
    }

    @Override
    public String getDescription() {
        return "Find resource manifests that have limits set beyond a specific maximum.";
    }

    @Override
    protected TreeVisitor<?, ExecutionContext> getSingleSourceApplicableTest() {
        if (fileMatcher != null) {
            return new HasSourcePath<>(fileMatcher);
        }
        return null;
    }

    @Override
    protected TreeVisitor<?, ExecutionContext> getVisitor() {
        ResourceLimit limit = new ResourceLimit(resourceLimit);
        YamlSearchResult result = new YamlSearchResult(randomId(), FindExceedsResourceValue.this,
                "exceeds maximum of " + limit.getValue());

        return new YamlIsoVisitor<ExecutionContext>() {
            @Override
            public Yaml.Scalar visitScalar(Yaml.Scalar scalar, ExecutionContext executionContext) {
                Cursor c = getCursor();
                if (((inLimits(resourceType, c) && "limits".equals(resourceValueType)) || (inRequests(resourceType, c) && "requests".equals(resourceValueType))) && asResourceLimits(scalar).getValue().exceeds(limit.getValue())) {
                    return scalar.withMarkers(scalar.getMarkers().addIfAbsent(result));
                }
                return super.visitScalar(scalar, executionContext);
            }
        };
    }
}

==> rewrite-kubernetes-1.15.1-sources.jar: org/openrewrite/kubernetes/resource/FindExceedsResourceValue.java <==
/*
 * Copyright 2021 the original author or authors.
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * <p>
 * https://www.apache.org/licenses/LICENSE-2.0
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.openrewrite.kubernetes.resource;

import lombok.EqualsAndHashCode;
import lombok.Value;
import org.openrewrite.*;
import org.openrewrite.internal.lang.Nullable;
import org.openrewrite.yaml.YamlIsoVisitor;
import org.openrewrite.yaml.tree.Yaml;

import static org.openrewrite.kubernetes.tree.K8S.ResourceLimits.inLimits;
import static org.openrewrite.kubernetes.tree.K8S.ResourceLimits.inRequests;
import static org.openrewrite.kubernetes.tree.K8S.asResourceLimits;

@Value
@EqualsAndHashCode(callSuper = true)
public class FindExceedsResourceValue extends Recipe {

    @Option(displayName = "Resource value type",
            description = "The type of resource value to search for.",
            example = "limits",
            valid = {"limits", "requests"})
    String resourceValueType;

    @Option(displayName = "Resource limit type",
            description = "The type of resource limit to search for.",
            example = "memory",
            valid = {"cpu", "memory"})
    String resourceType;

    @Option(displayName = "Resource limit",
            description = "The resource limit maximum to search for to find resources that request more than the maximum.",
            example = "2Gi")
    String resourceLimit;

    @Option(displayName = "Optional file matcher",
            description = "Matching files will be modified. This is a glob expression.",
            required = false,
            example = "**/pod-*.yml")
    @Nullable
    String fileMatcher;

    @Override
    public String getDisplayName() {
        return "Find exceeds resource limit";
    }

    @Override
    public String getDescription() {
        return "Find resource manifests that have limits set beyond a specific maximum.";
    }

    @Override
    protected TreeVisitor<?, ExecutionContext> getSingleSourceApplicableTest() {
        if (fileMatcher != null) {
            return new HasSourcePath<>(fileMatcher);
        }
        return null;
    }

    @Override
    protected TreeVisitor<?, ExecutionContext> getVisitor() {
        ResourceLimit limit = new ResourceLimit(resourceLimit);
        String result = "exceeds maximum of " + limit.getValue();

        return new YamlIsoVisitor<ExecutionContext>() {
            @Override
            public Yaml.Scalar visitScalar(Yaml.Scalar scalar, ExecutionContext executionContext) {
                Cursor c = getCursor();
                if (((inLimits(resourceType, c) && "limits".equals(resourceValueType)) || (inRequests(resourceType, c) && "requests".equals(resourceValueType))) && asResourceLimits(scalar).getValue().exceeds(limit.getValue())) {
                    return scalar.withMarkers(scalar.getMarkers().searchResult(result));
                }
                return super.visitScalar(scalar, executionContext);
            }
        };
    }
}
